/* Created from create_recycle (which was from create) */
/* output from beaglesim + simpop_fgl -> haplotypes    */

/* TO DO : 
-- Get info from Metadata file
-- Recycling inputs optional 
-- Spaces and labels not optional
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "simu.h"

extern int labelcounter;

int main(int argv, char *argc[])
{

	/* initialise */
    int nout, nsnps, nhaps, maxrow, nice_output;
    FILE * sim_chroms;
    FILE * snps_file;
    FILE * metadata;
    FILE * which_indivs_file;
    int i,j,k;
    Person * people;
    char ** snps;
    int * snp_pos, * which_indivs;
    int nextindex=0;
    Hashnode ** mytable;
    Chromseglist * curr;
    int fgl_dirty;
    int snp_array_col;

    if (argv!=6) 
    {
        fprintf(stdout,"Usage: [simulated_fgl_file] [founderhap_file] [SNPs_metadata_file] [Which_to_create] [Output with spaces and labels?]\n");
        return 1;
    }

	/* output */
nice_output = atoi( argc[5] );

	/* pedigree */
    sim_chroms = fopen(argc[1],"r");
    if (!sim_chroms)
    {
        fprintf(stderr,"Error: File %s not found\n",argc[1]);
        return 1;
    }
	/* founder haps */
    snps_file = fopen(argc[2],"r");
    if (!snps_file)
    {
        fprintf(stderr,"Error: File %s not found\n",argc[2]);
        return 1;
    }
	/* metadata */
    metadata = fopen(argc[3],"r");
    if (!metadata)
    {
        fprintf(stderr,"Error: File %s not found\n",argc[3]);
        return 1;
    }
    fscanf(metadata,"%d %*s",&nhaps); /*num input haplotypes*/
    fscanf(metadata,"%d %*s",&nsnps);    /*num snps*/
    snp_pos = malloc(nsnps*sizeof(int));
    for (i=0;i<nsnps;i++)
    {
        fscanf(metadata,"%d",&snp_pos[i]);/* snp locations*/
    }

	/* recycling */	

    	which_indivs_file = fopen( argc[4], "r"); /* assume we have a file */

	/*if there is no file*/
    	if(which_indivs_file == 0)
	{
		fprintf(stderr,"Create order file not found\n");
		
		nout = atoi( argc[4] );
		if( nout == 0 )
		{
		fprintf(stderr, "Error: If no create order file, must input #to print\n");
		return 1;
		}
		else
		{
		fprintf(stderr,"Will print individuals 1 to %d\n", nout);		
		/*create which_indivs vector */
		which_indivs = malloc(nout*sizeof(int));
		for (i=0;i<nout;i++)
    		{
        	which_indivs[i]=i; 
    		}
		maxrow = nout;
		}
    	}	
    else /*use the file*/
    {	
	fprintf(stderr, "Reading in %s ...\n", argc[4] );
    	nout = 0;/*nout from file*/
	maxrow = 0;
    	while(EOF!=fscanf(which_indivs_file,"%*d")) nout++;   	
    	rewind(which_indivs_file); /*reset file pointer */
	which_indivs = malloc(nout*sizeof(int));
    	for (i=0;i<nout;i++)
    	{
        	fscanf(which_indivs_file,"%d",which_indivs+i);
        	which_indivs[i]--; /* switch to 0-index */
		if( which_indivs[i] > maxrow) /*update max row*/
		{
		maxrow = which_indivs[i]+1;
		}
    	}
    }


    /* read in the REAL snps */
    fprintf(stderr,"Reading in SNP data...\n");
    snps = populate_snps_new(nsnps,nhaps,snps_file);


    /* read in ALL the SIMULATED chromosomes from simulation output up to maxrow */
    fprintf(stderr,"Reading in simulated chromosomes...\n");
    people = malloc(nout*sizeof(Person));
    for (i=0;i<maxrow*2;i++)
    {
        people[i] = read_person(sim_chroms);
    }

    mytable = make_hashtable();

/* this will become the output */
    fprintf(stderr,"Creating new haplotypes...\n");

    for (i=0;i<nout;i++)
    /* loop through output INDIVIDUALS */
    {
        for (j=0;j<2;j++)
        /* loop through each chromosome */
        {
            if(nice_output) printf("%d ",people[which_indivs[i]].id);

            /* which chromosome are we doing? */
            curr = j ? people[which_indivs[i]].paternal : people[which_indivs[i]].maternal;
            fgl_dirty = 1;
            for (k=0;k<nsnps;k++)
            /* loop through SNPs */
            {
                while(curr->current->endpos<snp_pos[k])
                {
                    /* fast forward until we're at the right place
                     * on the simulated chromosome; if we move,
                     * we need to look up the new FGL */
                    curr = curr->next;
                    if (!curr)
                    {
                        fprintf(stderr,"The simulated chromosome is too short! Marker %d at position %d not covered by an FGL.\n",k,snp_pos[k]);
                        return 1;
                    }
                    fgl_dirty = 1;
                }
                if (fgl_dirty)
                {
                    /* if we need to check the FGL, look it up in the 
                     * table and possibly update the value there */
                    snp_array_col = get_index(curr->current->fgl,mytable,&nextindex);
                    fgl_dirty = 0;
                    if (snp_array_col >= nhaps)
                    {
                        fprintf(stderr,"not enough real founder SNPs!\n");
                        return 1;
                    }
                }
                /* finally, print the char */
                printf("%c",snps[k][snp_array_col]);
                if(nice_output) printf(" ");
            }
            printf("\n");
        }
    }

    return 0;
}

